#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.control.mov_caixa import MovCaixa
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.models.cad_operadores_data import CaixaCadOperadores
from sigpdv.models.db_utils import exec_sql
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.control.utils import atualizar_saldo_inicial_supervisor
from sigpdv.ui.messagebox import MSG_INFO, NO_ROLE, MSG_QUESTION, message_dlg, YES_ROLE
from datetime import datetime


class AberturaFechamentoTurno(object):
    def __init__(self, parent):
        self.parent = parent

    @Log
    def abrir_turno(self, loja, caixa, cod_operador):
        """
        Realiza a abertura do turno para o operador informado.
        """
        if self.parent.verifica_z_emitido():
            message_dlg(self.parent, MSG_INFO, 'Redução Z de hoje já foi emitida!\n'
                                        'Não é possivel abrir turno, deve-se esperar o próximo'
                                        ' dia fiscal!')
            return False

        cad_operador = CaixaCadOperadores.get_by(supope_codigo=cod_operador)
        msg = 'Ola %s.\n\nDeseja abrir um novo turno?' % str(cad_operador.name)
        if message_dlg(self.parent, MSG_QUESTION, msg) == NO_ROLE:
            return False

        data_hora_abertura = datetime.now()
        try:
            self.parent.mov_caixa = MovCaixa().create(
                dict(supcax_loja=loja,
                     supcax_caixa=caixa,
                     supcax_data_hora_abertura=data_hora_abertura,
                     supcax_operador_id=cad_operador.id,
                     supcax_fechado='A',
                     supcax_enviado='N'))

            if not self.parent.mov_caixa:
                return False

            self.parent.ui.lblNumCaixa.setText(u'%03d <b>Aberto' % caixa)
            self.parent.ui.mainMenu.item(1).setText(u'Fechar Turno')  # imenu_turno
        except Exception as err:
            Log.error(str(err))

        message_dlg(self.parent, MSG_INFO,
                    'Turno aberto:\n\nOperador: %s\nCod. Cx.: %d\nLoja: %.4d\nData/Hora: %s'
                    % (str(cad_operador.name), caixa, loja, data_hora_abertura))

        return True

    @Log
    def fechar_turno(self, mov_caixa, loja, caixa, operador=(0, '')):
        """
        Realiza o fechamento de turno e imprime um relatório gerencial.

        :param mov_caixa_id: id do turno
        :param loja: loja_id
        :param caixa: caixa_id
        :param operador: tupla com o id e o nome do operador
        :return: boolean
        """
        cupom_aberto = self.parent.obj_cupom_fiscal.get_cupom_aberto()
        if cupom_aberto:
            self.parent.mov_capa = cupom_aberto
            self.parent.cancelar_nota_fiscal(automatico=True)

        texto = 'Relatório: Fechamento de Turno'
        if self.parent.teclado:
            self.parent.teclado.envia_display('Imprimindo %s...' % texto)

        self.parent.show_info_proccess(True, texto)
        try:
            # todo: Criar o cupom de fechamento de turno
            data_hora_fechamento = datetime.now()
            text = self.get_valores_fechamento_turno(mov_caixa, loja, caixa, operador,
                                                     data_hora_fechamento.strftime('%H:%M:%S'))
            if not text:
                raise Exception('Erro ao fechar o turno. Verifique se o turno já foi fechado.')

            # GerencialReport(report_id=config['rg_relatorio_gerencial'],
            #                 text=text,
            #                 loja_id=loja,
            #                 caixa_id=caixa,
            #                 operador_id=operador[0],
            #                 tipo='F',
            #                 parent=self.parent)
            #
            self.parent.obj_impressoras.supcci_imprimir(text)
            MovCaixa().write(mov_caixa,
                             {'supcax_data_hora_fechamento': data_hora_fechamento,
                              'supcax_fechado': 'F', 'supcax_enviado': 'N'})

            self.parent.show_info_proccess(False)

            self.parent.ui.mainMenu.item(1).setText(u'Abrir Turno')  # imenu_turno
            data_fechamento = self.parent.mov_caixa.supcax_data_hora_abertura.strftime('%d/%m/%Y')
            message_dlg(self.parent, MSG_INFO, "O turno do dia: %s foi fechado!" % str(data_fechamento))
            self.parent.mov_caixa = False

        except Exception as err:
            self.parent.show_info_proccess(False)
            Log.error(str(err))
            return False

        return True

    def get_saldo_turno(self, mov_caixa, loja, caixa, operador=(0, '')):
        """
        Retorna o saldo em caixa para o turno informado.

        :param mov_caixa: turno
        :param loja: loja_id
        :param caixa: caixa_id
        :param operador: tupla com o id e o nome do operador
        :return: float com o valor do saldo em caixa
        """
        if not mov_caixa:
            return 0

        saldo = self.get_valores_fechamento_turno(mov_caixa, loja, caixa, operador, get_saldo=True)
        return saldo

    @Log
    def get_valores_fechamento_turno(self, mov_caixa, loja, caixa, operador=(0, ''),
                                     hora_fechamento='', get_saldo=False):
        """
        Se get_saldo = False, retorna o texto à ser impresso no relatório gerencial
        de fechamento de turno.
        Se get_saldo = True, retorna o valor total do saldo em caixa para o turno informado.
        """

        data_mov = MovCaixa().get_data_hora_abertura(loja, caixa, mov_caixa.id)

        if not data_mov:
            return False

        operador = CaixaCadOperadores.get_by(id=mov_caixa.supcax_operador_id)
        if isinstance(operador, list):
            operador = operador[0]

        operador = (operador.id, operador.name or '')
        totais_dinheiro = 0
        qtds_dinheiro = 0

        totais_cheque = 0
        qtds_cheque = 0

        totais_cartao = 0
        qtds_cartao = 0

        totais_pix = 0
        qtds_pix = 0

        totais_outros = 0
        qtds_outros = 0

        texto = "\n"
        texto += ("LOJA: %04d CAIXA: %03d" % (config['loja'], config['caixa'])).ljust(48) + '\n'
        sql = """
      SELECT SUM(totvalor) AS totvalor,
             COUNT(qtdfin) AS qtdfin,
             tipo_finalizadora
      FROM (SELECT cmf.supfec_valor - cmf.supfec_troco AS totvalor,
                   cmf.supfec_tipo_finalizadora        AS qtdfin,
                   CASE WHEN ccf.name ilike '%PIX%'
                        THEN 'T'
                        ELSE cmf.supfec_tipo_finalizadora 
                   END                                 AS tipo_finalizadora,
                   cmc.supcap_tipo
            FROM caixa_mov_fechamentos        AS cmf
            INNER JOIN caixa_cad_finalizadora As ccf ON ccf.supfin_codigo = cmf.supfec_finalizadora
            INNER JOIN caixa_mov_capa         AS cmc ON cmc.id = cmf.supfec_capa_id
            WHERE cmc.supcap_data_fim > '{data_mov}'
              AND (cmc.supcap_state IN ('3', '4', '6', '2E') 
                   OR cmc.supcap_state IS NULL)
              AND cmc.supcap_tipo IN ('V', 'D', 'W', 'R')
              AND cmc.supcap_loja = {loja}
              AND cmc.supcap_caixa = {caixa}) AS tmp
      GROUP BY tipo_finalizadora, supcap_tipo
            
        """.format(data_mov=data_mov,
                   loja=loja,
                   caixa=caixa)

        for dados in exec_sql(sql, 'C', ret=True):
            if dados.get('tipo_finalizadora') == 'D':
                totais_dinheiro += (dados.get('totvalor') or 0)
                qtds_dinheiro += (dados.get('qtdfin') or 0)

            elif dados.get('tipo_finalizadora') == 'H':
                totais_cheque += (dados.get('totvalor') or 0)
                qtds_cheque += (dados.get('qtdfin') or 0)

            elif dados.get('tipo_finalizadora') == 'C':
                totais_cartao += (dados.get('totvalor') or 0)
                qtds_cartao += (dados.get('qtdfin') or 0)

            elif dados.get('tipo_finalizadora') == 'T':
                totais_pix += (dados.get('totvalor') or 0)
                qtds_pix += (dados.get('qtdfin') or 0)

            elif dados.get('tipo_finalizadora') == 'O':
                totais_outros += (dados.get('totvalor') or 0)
                qtds_outros += (dados.get('qtdfin') or 0)

        ############################################################################################

        texto += ("Data Mov.  : %s" % data_mov.strftime('%d/%m/%Y')).ljust(48) + '\n'
        texto += ("Operador   : %03d-%s" % operador).ljust(48) + '\n'
        texto += ("Abertura   : %s" % data_mov.strftime('%H:%M:%S')).ljust(48) + '\n'
        texto += ("Fechamento : %s" % str(hora_fechamento)).ljust(48) + '\n'
        texto += "\n"
        texto += ("* %s" % 'Vendas Liquidas').ljust(48) + '\n'
        texto += "------------ --- ------------".ljust(48) + '\n'
        texto += "Finalizadora Qtd        Valor".ljust(48) + '\n'
        texto += "------------ --- ------------".ljust(48) + '\n'

        texto += ("Dinheiro     %03d %12.2f" % (qtds_dinheiro, totais_dinheiro)).ljust(48) + '\n'
        texto += ("Cheque       %03d %12.2f" % (qtds_cheque, totais_cheque)).ljust(48) + '\n'
        texto += ("Cartao       %03d %12.2f" % (qtds_cartao, totais_cartao)).ljust(48) + '\n'
        texto += ("PIX          %03d %12.2f" % (qtds_pix, totais_pix)).ljust(48) + '\n'
        texto += ("Outros       %03d %12.2f" % (qtds_outros, totais_outros)).ljust(48) + '\n'

        # ----------------------------------- Total das Vendas ----------------------------------- #
        soma = sum([totais_dinheiro,
                    totais_cheque,
                    totais_cartao,
                    totais_pix,
                    totais_outros])

        texto += "-----------------------------".ljust(48) + '\n'
        texto += ("Total de Vendas: %12.2f" % soma).ljust(48) + '\n'
        texto += "\n"

        ############################################################################################

        # ----------------------------------- Sangria -------------------------------------------- #
        sql = """

            SELECT SUM(cmf.supfec_valor)               AS totvalor,
                   COUNT(cmf.supfec_tipo_finalizadora) AS qtdfin,
                   cmf.supfec_tipo_finalizadora        AS tipo_finalizadora
            FROM caixa_mov_fechamentos AS cmf
            INNER JOIN caixa_mov_capa  AS cmc ON cmc.id = cmf.supfec_capa_id
            WHERE cmc.supcap_data_fim > '{data_mov}'
              AND cmc.supcap_tipo = 'A'
              AND cmc.supcap_loja={loja}
              AND cmc.supcap_caixa={caixa}
            GROUP BY cmf.supfec_tipo_finalizadora
        """.format(data_mov=data_mov,
                   loja=loja,
                   caixa=caixa)

        total_sd = 0
        total_sh = 0
        total_sc = 0
        totais_sp = 0 # pix
        total_so = 0
        qtda_sd = 0
        qtda_sh = 0
        qtda_sc = 0
        qtda_sp = 0
        qtda_so = 0
        texto += "* SANGRIAS".ljust(48) + '\n'
        texto += "------------ --- ------------".ljust(48) + '\n'
        texto += "Finalizadora Qtd        Valor".ljust(48) + '\n'
        texto += "------------ --- ------------".ljust(48) + '\n'
        for dados in exec_sql(sql, 'C', ret=True):
            if dados.get('tipo_finalizadora') == 'D':
                total_sd = dados.get('totvalor') or 0
                qtda_sd = dados.get('qtdfin') or 0

            elif dados.get('tipo_finalizadora') == 'H':
                total_sh = dados.get('totvalor') or 0
                qtda_sh = dados.get('qtdfin') or 0

            elif dados.get('tipo_finalizadora') == 'C':
                total_sc = dados.get('totvalor') or 0
                qtda_sc = dados.get('qtdfin') or 0

            elif dados.get('tipo_finalizadora') == 'T':
                totais_sp = dados.get('totvalor') or 0
                qtda_sp = dados.get('qtdfin') or 0

            elif dados.get('tipo_finalizadora') == 'O':
                total_so = dados.get('totvalor') or 0
                qtda_so = dados.get('qtdfin') or 0

        texto += ("Dinheiro     %03d %12.2f" % (qtda_sd, total_sd)).ljust(48) + '\n'
        texto += ("Cheque       %03d %12.2f" % (qtda_sh, total_sh)).ljust(48) + '\n'
        texto += ("Cartao       %03d %12.2f" % (qtda_sc, total_sc)).ljust(48) + '\n'
        texto += ("PIX          %03d %12.2f" % (qtda_sp, totais_sp)).ljust(48) + '\n'
        texto += ("Outros       %03d %12.2f" % (qtda_so, total_so)).ljust(48) + '\n'

        # ---------------------------------- Total das Sangrias ---------------------------------- #
        texto += "-----------------------------".ljust(48) + '\n'
        texto += ("Total Sangrias.: %12.2f" % sum([total_sd, total_sh, total_sc, totais_sp,
                                                   total_so])).ljust(48) + '\n'
        texto += "\n"

        # -------------------------------------- Suprimento -------------------------------------- #
        sql = """

            SELECT SUM(supcap_total_cupom) AS totalcupom,
                   COUNT(supcap_nro_cupom) AS qtdsup
            FROM  caixa_mov_capa
            WHERE supcap_data_fim > '%s'
                  AND supcap_tipo='U'
                  AND supcap_loja=%d
                  AND supcap_caixa=%d

        """ % (data_mov, loja, caixa)

        ret_sql_su = exec_sql(sql, 'C', ret=True)

        total_su = ret_sql_su[0].get('totalcupom') if ret_sql_su[0].get('totalcupom') else 0
        qtda_su = ret_sql_su[0].get('qtdsup')

        texto += ("* SUPRIMENTO (%03d) %10.2f" % (qtda_su, total_su)).ljust(48) + '\n'

        # -------------------------------------- Contra-Vale ------------------------------------- #
        sql = """

              SELECT SUM(supcap_total_cupom) AS totvalor,
                     COUNT(supcap_loja)      AS qtdfin
              FROM   caixa_mov_capa
              WHERE  supcap_tipo = 'O'
                     AND supcap_loja = %d
                     AND supcap_caixa = %d
                     AND supcap_data_fim > '%s'

        """ % (loja, caixa, data_mov)

        ret_sql_cv = exec_sql(sql, 'C', ret=True)

        total_cv = ret_sql_cv[0].get('totvalor') if ret_sql_cv[0].get('totvalor') else 0
        qtda_cv = ret_sql_cv[0].get('qtdfin')

        texto += ("* CONTRA-VALE(%03d) %10.2f" % (qtda_cv, total_cv)).ljust(48) + '\n'
        dados_ficha_doacao = self.get_dados_ficha_doacao(data_mov, loja, caixa)
        if dados_ficha_doacao:
            if dados_ficha_doacao.get(config['tot_ficha']):
                texto += ("* Ficha " + ("%10.2f" % dados_ficha_doacao.get(
                                                                     config['tot_ficha'])).rjust(22)).ljust(48) + '\n'
            if dados_ficha_doacao.get(config['tot_devolucao_ficha']):
                texto += ("* Devolucao Ficha " + ("%10.2f" % dados_ficha_doacao.get(
                                                           config['tot_devolucao_ficha'])).rjust(12)).ljust(48) + '\n'
            if dados_ficha_doacao.get(config['totalizador_doacao']):
                texto += ("* Doacao " +  ("%10.2f" % dados_ficha_doacao.get(
                                                            config['totalizador_doacao'])).rjust(21)).ljust(48) + '\n'

        texto += "\n"

        # ------------------------------------ Saldo em Caixa ------------------------------------ #
        tot_d = totais_dinheiro + total_su - total_sd
        tot_h = totais_cheque - total_sh
        tot_c = totais_cartao - total_sc
        tot_t = totais_pix - totais_sp
        tot_o = totais_outros - total_so

        tot_geral = sum([tot_d, tot_h, tot_c, tot_t, tot_o, total_cv])

        texto += "******* SALDO EM CAIXA ******".ljust(48) + '\n'
        texto += "\n"

        if tot_geral:
            if tot_d:
                texto += ("Dinheiro...: %16.2f" % tot_d).ljust(48) + '\n'
            if tot_h:
                texto += ("Cheque.....: %16.2f" % tot_h).ljust(48) + '\n'
            if tot_c:
                texto += ("Cartao.....: %16.2f" % tot_c).ljust(48) + '\n'
            if tot_t:
                texto += ("PIX........: %16.2f" % tot_t).ljust(48) + '\n'
            if tot_o:
                texto += ("Outros.....: %16.2f" % tot_o).ljust(48) + '\n'

            texto += "\n"
            texto += ("TOTAL GERAL: %16.2f" % tot_geral).ljust(48) + '\n'
        else:
            texto += "******* SALDO EM CAIXA ******".ljust(48) + '\n'
            texto += "\n"
            texto += "::: NAO HA SALDO EM CAIXA :::".ljust(48) + '\n'

        texto += "\n"
        texto += "*****************************".ljust(48) + '\n'
        texto += "\n"

        if get_saldo:
            res = tot_geral
        else:
            res = str(texto)

        return res

    def get_dados_ficha_doacao(self, data_mov, loja, caixa):
        sql = """
                SELECT SUM(cmc.supcap_total_cupom)   AS total, 
                       cmc.supcap_cod_tot_nao_fiscal AS totalizador
                FROM caixa_mov_capa  AS cmc
                WHERE cmc.supcap_data_fim > '{data_mov}'
                  AND cmc.supcap_tipo = '{tipo}'
                  AND cmc.supcap_loja = {loja}
                  AND cmc.supcap_caixa = {caixa}
                  AND cmc.supcap_cod_tot_nao_fiscal IN ({tot})
                GROUP BY cmc.supcap_cod_tot_nao_fiscal
        """.format(tipo='W',
                   data_mov=data_mov,
                   loja=loja,
                   caixa=caixa,
                   tot= '%s, %s, %s' % (config['tot_ficha'] or 0,
                                        config['tot_devolucao_ficha'] or 0,
                                        config['totalizador_doacao'] or 0))
        res = exec_sql(sql, 'C', ret=True)
        if not res:
            return False

        vals = {}
        map(lambda x: vals.update({x.get('totalizador'): x.get('total')}), res)
        return vals


