# -*- coding: utf-8 -*-

__autor__ = 'Dener Rodrigues'

import xmlrpclib
import psycopg2
import psycopg2.extras
from datetime import datetime, date
from sqlalchemy import create_engine
from sqlalchemy.schema import MetaData, ThreadLocalMetaData
from sqlalchemy.orm import scoped_session, sessionmaker
from sigpdv.tools.rpc_timeout import ServerProxy
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log

# -------------------------------------------- Caixa --------------------------------------------- #
connection_caixa = "postgresql+psycopg2://%s:%s@%s/%s" % (config['db_pdv_user'],
                                                          config['db_pdv_pass'],
                                                          config['db_pdv_host'],
                                                          config['db_pdv_name'])

a_engine = create_engine(connection_caixa, echo=False,
                         logging_name='%s:%s' % (config['db_pdv_host'], config['db_pdv_name']))

a_session = scoped_session(sessionmaker(autoflush=True, autocommit=True, bind=a_engine))

a_metadata = ThreadLocalMetaData()
a_metadata.bind = a_engine

sess = a_session

# ------------------------------------------ Supervisor ------------------------------------------ #
connection_supervisor = "postgresql+psycopg2://%s:%s@%s:%s/%s" % (config['db_sup_user'],
                                                                  config['db_sup_pass'],
                                                                  config['db_sup_host'],
                                                                  config['pg_sup_port'],
                                                                  config['db_sup_name'])
b_engine = create_engine(connection_supervisor, echo=False, pool_timeout=5,
                         logging_name='%s:%s' % (config['db_sup_host'], config['db_sup_name']),
                         connect_args={'connect_timeout': 5})

b_session = scoped_session(sessionmaker(autocommit=True, bind=b_engine))
b_metadata = MetaData()
b_metadata.bind = b_engine
b_sess = b_session


# ------------------------------------------------------------------------------------------------ #


class ErroConexao(Exception):
    """Base"""


def fetchall_2_dictfetchall(cr):
    """
    fetchall -> dictfetchall. Essa funcao converte o resultado do fectchall de um cursor
    para dictfetchall. O psycopg2 nao possui a funcionalidade dictfetchall, por isso a
    conversao.
    """

    desc = cr.description
    fetchall = cr.fetchall()
    res = [dict(zip([col[0] for col in desc], row)) for row in fetchall]

    return res


def get_connection(caixa_supervisor_erp='C', statement_timeout=0):
    if caixa_supervisor_erp.upper() in ['C', 'CAIXA']:
        connection = psycopg2.connect(database=config['db_pdv_name'], user=config['db_pdv_user'],
                                      password=config['db_pdv_pass'], host=config['db_pdv_host'],
                                      port=5432)

    elif caixa_supervisor_erp.upper() in ['S', 'SUPERVISOR']:
        host = config['db_sup_host']
        try:
            connection = psycopg2.connect(database=config['db_sup_name'], user=config['db_sup_user'],
                                          password=config['db_sup_pass'], host=host,
                                          port=config['pg_sup_port'],
                                          connect_timeout=5,
                                          options='-c statement_timeout=%s' % statement_timeout)
        except Exception as err:
            connection = False
            msg = '(%s, %s)' % (config['db_sup_name'], host)
            Log.error('Falha ao obter conexão com o banco de dados%s! \n%s' % (msg, str(err)))

    elif caixa_supervisor_erp.upper() in ['E', 'ERP']:
        host = config['pg_server']
        try:
            connection = psycopg2.connect(database=config['pg_db'], user=config['pg_user'],
                                          password=config['pg_pass'], host=config['pg_server'],
                                          port=config['pg_port'])
        except Exception as err:
            msg = '(%s, %s)' % (config['pg_db'], host)
            Log.error('Falha ao obter conexão com o banco de dados%s! \n%s' % (msg, str(err)))
            connection = False
    else:
        connection = False

    return connection


def remove_accents(text, cod='utf8'):
    from unicodedata import normalize, category

    try:
        unicode_string = str(text)
    except UnicodeEncodeError:
        res = normalize('NFKD', text).encode('ASCII', 'ignore')

    else:
        try:
            res = unicode(filter(lambda c: category(c) != 'Mn',
                                 normalize('NFKD', unicode_string.decode(cod, 'replace'))))
        except:
            res = normalize('NFKD', text.decode('utf8')).encode(cod, 'ignore')

    return res


def exec_sql(sql=None, caixa_supervisor_erp='C', ret=True, connection=None, close_connection=True,
             auto_commit=True):
    '''

    :param sql: sql que deseja executar
    :param caixa_supervisor_erp: 'C' = banco de dados do caixa, 'S' banco de dados do supervisor 'E' banco do ERP
    :param ret: True = retorna o resultado do sql, False = sem retorno
    :param connection:
    :param close_connection:
    :param auto_commit:
    :return:
    '''

    try:
        # Realiza a conexão no banco de dados
        if not connection:
            connection = get_connection(caixa_supervisor_erp)
    except Exception as e:
        raise e

    cr = False
    try:
        if not connection:
            raise ErroConexao('Falha ao obter conexão com o banco de dados("%s")!' %
                              caixa_supervisor_erp)
        # Prepara um objeto cursor usando o método cursor()
        cr = connection.cursor(cursor_factory=psycopg2.extras.DictCursor)

        # Executar a query
        cr.execute(sql)

        if ret:  # Se tiver retorno, converte fetchall -> dictfetchall
            try:
                res = fetchall_2_dictfetchall(cr)
            except:
                res = True
        else:
            res = True

        if auto_commit:
            # Commit em suas alterações no banco de dados
            cr.connection.commit()

        # Fecha a conexão com o banco de dados
        if close_connection:
            connection.close()
    except Exception as err:
        if auto_commit and cr:
            # Rollback em caso de erro
            cr.connection.rollback()
        if close_connection and connection:
            connection.close()
        raise err

    return res


def testa_conexao_rpc(server, username, pwd, dbname, port=8069):
    retval = dict(res=True,
                  msg="Conexão efetuado com sucesso para '%s' "
                      "usando o banco '%s:%s'" % (str(username), str(server), str(dbname)))

    try:
        sock_common = ServerProxy('http://%s:%d/xmlrpc/common' % (server, port))
        uid = sock_common.login(dbname, username, pwd)
        sock = ServerProxy('http://%s:%d/xmlrpc/object' % (server, port))
        try:
            sock.execute(dbname, uid, pwd, 'valida.pendencias', 'valpen_verifica_versao', [])
        except Exception as e:
            if "has no attribute 'valpen_verifica_versao'" not in str(e):
                raise e

        del sock
    except xmlrpclib.Fault as e:
        retval = dict(res=False, msg='%s' % e.faultCode)
    except Exception as e:
        retval = dict(res=False, msg='%s' % e)

    return retval


def testa_conexao_pg(db, user, host, pwd, port):
    string_conexao = "dbname='%s' connect_timeout=10 user='%s' host='%s' password='%s'" \
                     "port='%s'" % (db, user, host, pwd, port)

    res = dict(res=True,
               msg="Conexão efetuado com sucesso para '%s' usando o banco '%s:%s' " \
                   "porta: %s" % (str(user), str(host), str(db), port))
    try:
        connection = psycopg2.connect(string_conexao)
        connection.close()
    except Exception as err:
        res = False
        msg = '(%s, %s)' % (db, host)
        Log.error('Falha ao obter conexão com o banco de dados%s! \n%s' % (msg, str(err)))

    return res


def dict_2_sql(table, data, _type='I', where='', tz_utc=False):
    """
    Converte um dicionario para uma sql.

    :param table: nome da tabela
    :param data: dados à serem manipulados
    :param _type: 'I' (Insert) - 'IU' (Insert Where Not Exists) - 'D' (Delete) - 'U' (Update)
    :param where: utilizado para realizar Update
    :param tz_utc: utilizado par gravar campos datetime no time-zone utc

    :return: retorna uma string com a sql pronta à ser executada
    """

    from sigpdv.control.utils import remove_accents

    def insert():
        sql = "INSERT INTO %s %s VALUES %s RETURNING id;" % (table, chave, valor)
        return sql

    def insert_where_not_exists():
        sql = """INSERT INTO %s
                             %s
                             SELECT %s WHERE NOT EXISTS (SELECT 1
                                                         FROM   %s
                                                         WHERE  %s = %s)
                 RETURNING id;""" % (table, chave, valor, table, chave, valor)

        sql = sql.replace("SELECT (", "SELECT ").replace(") WHERE NOT", " WHERE NOT")
        return sql

    def insert_on_duplicate_k_up():
        key_up = [tuple('%s="%s"' % (k, remove_accents(v.encode('utf-8'))
        if isinstance(v, (unicode, str)) else v)
                        for k, v in data.iteritems())]

        sql = "INSERT INTO %s %s VALUES %s ON DUPLICATE KEY UPDATE %s;" % (
            table,
            str(data_keys[0]).replace("'", ""),
            str(data_value[0]).replace("'None'", 'null').replace('%', '').replace("u'", "'"),
            str(key_up)[2:-2].replace("u'", '').replace("'", "").replace('"None"', 'null'))

        return sql

    def delete():
        sql = "DELETE FROM %s WHERE %s = %s;" % (table, chave, valor)
        return sql

    def update():
        u = ''
        for campo in data:
            if u:
                u += ', '
            if isinstance(data.get(campo), (unicode, str)):
                u += "%s='%s'" % (campo, remove_accents(data.get(campo)))
            elif tz_utc and type(data.get(campo)) is datetime:
                u += "%s='%s' at time zone 'UTC'" % (campo, data.get(campo))
            else:
                u += "%s=%s" % (campo, data.get(campo))

        key = [tuple('%s' % k for k in where.keys())]

        vlr = [tuple('%s' % remove_accents(v)
                     if isinstance(v, (unicode, str)) else v for v in where.values())]

        w = '%s=%s' % (str(key).replace("[(u'", "(").replace("',)]", ")").replace("u'", "'")
                       .replace("'", "").replace(")]", ")").replace("[(", "("),
                       str(vlr).replace("Decimal('", "'").replace(")]", ")").replace("[(", "(")
                       .replace("u'", "'").replace('None', 'null')
                       .replace(",)", ")"))

        sql = "UPDATE %s SET %s WHERE %s RETURNING id;" % (table, u.replace('None', 'null'), str(w))
        return sql

    try:
        data_keys = [tuple('%s' % k for k in data.keys())]
        lista = []

        for data_value in data.values():
            if isinstance(data_value, (unicode, str)):
                if 'xml' in data_value and '\'' in data_value:
                    data_value = data_value.replace('\'', "''")

                data_value = "'%s'" % remove_accents(data_value)
            elif tz_utc and type(data_value) is datetime:
                utc = "('%s' at time zone 'UTC')" % data_value
                data_value = utc
            elif type(data_value) in (datetime, date):
                data_value = "'%s'" % data_value
            else:
                data_value = '%s' % data_value

            if 'xml' not in data_value:
                data_value = data_value.replace('False', "'F'").replace('True', "'T'")

            lista.append(data_value)

        data_value = tuple(lista)

        chave = str(data_keys).replace("u'", "'").replace("'", "").replace(')]', ')').replace('[(', '(')

        valor = '(%s)' % (', '.join(lista)).replace("'None'", 'null').replace("None", 'null')

    except Exception as e:
        Log.error(str(e))

    try:
        if _type == 'D':
            ret = delete()

        elif _type == 'U':
            ret = update()

        elif _type == 'I':
            ret = insert()

        elif _type == 'IU':
            ret = insert_where_not_exists()

        else:
            ret = insert_on_duplicate_k_up()
    except Exception as err:
        ret = False
        Log.error(str(err))

    return ret


modulo_instalado_sorteio = modulo_instalado_fichas = modulo_instalado_producao_comanda = \
    modulo_instalado_promocoes = False
modulos = ['supervisor_fichas', 'supervisor_producao_comanda', 'supervisor_promocoes']
try:
    res_sql = exec_sql("""SELECT name 
                          FROM ir_module_module 
                          WHERE name in ('%s')
                            AND state = 'installed';
                         """ % "','".join(modulos), 'S', True)
    if res_sql:
        for i in res_sql:
            if i.get('name') == 'supervisor_fichas':
                modulo_instalado_fichas = True
            elif i.get('name') == 'supervisor_producao_comanda':
                modulo_instalado_producao_comanda = True
            elif i.get('name') == 'supervisor_promocoes':
                modulo_instalado_promocoes = True

except Exception as e:
    Log.error('Erro ao verificar modulos instalados: %s' % e)
