# -*- coding: utf-8 -*-

from PySide import QtCore, QtGui
from ctypes import CDLL, c_ushort, c_char_p, c_int, c_buffer, byref, sizeof
from decimal import Decimal
from sigpdv.ui.messagebox import YES_ROLE, MSG_ERROR, MSG_QUESTION, MSG_INFO, message_dlg
from sigpdv.ui.dlpPwd import dlg_pwd
from sigpdv import root_path as ROOT_PATH
from sigpdv.tools.config import config
from sigpdv.models.cad_bandeiras_data import CaixaCadBandeiras
from sigpdv.models.cad_finalizadora_data import CaixaCadFinalizadora
from sigpdv.models.cor_nsu_data import CaixaCorNsu
from sigpdv.models.db_utils import exec_sql
from sigpdv.tools.log import traced as Log
from sigpdv.sigtef.recarga_celular import RecargaCelular
from sigpdv.control.relatorio_gerencial import GerencialReport
from datetime import datetime
import platform
import os

lib = 'lib_x86_64' if platform.machine() == 'x86_64' else 'lib'
path_lib = os.path.join(ROOT_PATH, 'sigtef', lib, 'libclisitef.so')

ERR_COD = {
     0: 'Negada pelo autorizador',
    -1: 'Módulo não inicializado',
    -2: 'Operação cancelada pelo operador',
    -3: 'Fornecida uma modalidade inválida',
    -4: 'Falta de memória para rodar a função',
    -5: 'Sem comunicação com o sitef',
    -6: 'Operação cancelada pelo usuário',
    -7: 'Reservado.',
    -8: 'A CliSiTef não possui a implementação da função necessária, '
        'provavelmente está desatualizada (a CliSiTefI é mais recente).',
    -9: 'A automação chamou a rotina ContinuaFuncaoSiTefInterativo sem '
        'antes iniciar uma função iterativa.',
    -10: 'Algum parâmetro obrigatório não foi passado pela automação comercial.',
    -12: 'Erro na execução da rotina iterativa. Provavelmente o processo iterativo '
         'anterior não foi finalizado até o final (enquanto o retorno for igual a 10000).',
    -15: 'Operação cancelada pela automação comercial.',
    -20: 'Parâmetro inválido passado para a função.',
    -21: 'Utilizada uma palavra proibida, por exemplo SENHA, para coletar dados em '
         'aberto no pinpad. Por exemplo na função ObtemDadoPinpadDiretoEx.',
    -25: 'Erro no Correspondente bancário: Deve realizar sangria.',
    -30: 'Erro de acesso ao arquivo. Certifique-se que o usuário que roda a aplicação tem '
         'direitos de leitura/escrita.',
    -40: 'Transação negada pelo SiTef',
    -41: 'Dados inválidos.',
    -42: 'Reservado.',
    -43: 'Falha no pinpad',
    -50: 'Transação não segura',
    -100: 'Erro interno no módulo',
    -110: 'Lista de produtos não atualizada / inválida',
    -111: 'Produto(s) sem promoção Sócio Torcedor',
    -114: 'Quantidade de produto não disponível',
    -115: 'Verificar o valor limite para o produto consultado',
}
TIPO_BANDEIRA = {1:'D', # debito
                 2:'C', # credito
                 3:'V', # voucher
                 }

CARTEIRA_DIGITAL = {'00150001': 'VC +',
                    '00190002': 'NuCash',
                    '00200003': 'Veloe',
                    '00210006': 'IzPay',
                    '00260005': 'Vee',
                    '00290006': 'Mercado Pago',
                    '00300007': 'MoneyPag',
                    '00340008': 'Troco Simples',
                    '00380009': 'Itaú',
                    '00580011': 'BrinksPay',
                    '00600012': 'PicPay',
                    '00650013': 'PagSeguro',
                    '60110024': 'Pix',
                    '80001000': 'Cielo',
                    '80021002': 'Tá Pago',
                    '80031003': '4All',
                    }

class SigTefI(object):
    """
    Classe SigTef
    Responsavel pela comunicação com o SiTef usando o modo
    interativo
    """

    def __init__(self, parent, configura_resultado=0, test=False, tipo=None):

        self.lib_clisitef = CDLL(path_lib)
        self.usando_sitef = False
        self.bloqueio = False # bloqueia a funcao inicia_sitef para nao ser executada 2 vezes
        self.desconto_socio_registrado = False
        self._test = test
        self._stop_event = None
        self._consulta_cheque = False
        self._status_iniciou_sitef = None
        self._transacoes = []
        self.tbl_nsu = CaixaCorNsu
        self._tef_nsu_socio_torcedor = None

        self.parent = parent
        self.gui = parent

        self.endereco_ip = c_char_p(config['servidor'])
        self.codigo_loja = c_char_p(config['empresa'])
        self.identificacao_terminal = c_char_p('cx%s%.3d' % (('%.4d' % config['loja'])[1:],
                                                             config['caixa']))
        self.configura_resultado = c_int(configura_resultado)
        if config['nfce_ambiente'] == 'P':
            self.parms_adicionoais = c_char_p('[ParmsClient=1=%s;2=08974572000183]' %
                                              self.parent.loja.supcfc_cnpj)
        else:  # homologacao
            self.parms_adicionoais = c_char_p('[ParmsClient=1=00000000000000;2=00000000000000]')

        self._configura_sitef()
        self.data_str = ''
        self.hora_str = ''
        self.proximo_comando = c_int(0)
        self.tipo_campo = c_int(0)
        self.tam_minimo = c_ushort(0)
        self.tam_maximo = c_ushort(0)
        self.buffer = c_buffer('', 20000)
        self.tam_max_buffer = c_int(20 * 1024 + 1)
        self.continua_navegacao = c_int(0)

        self.modalidade = c_int(0)
        self.valor = c_char_p('')
        self.numero_cupom_fiscal = c_char_p('')
        self.data_fiscal = c_char_p('')
        self.horario = c_char_p('')
        self.operador = c_char_p('')
        self.restricoes = c_char_p('')
        self._cbarras = False

        # FINALIZADORAS
        self.finalizadora_id = None
        self.nome_finalizadora = None
        self.tipo_finalizadora = tipo
        self.mod_pgto_grupo = None
        self.mod_pgto_subgrupo = None
        self.cod_instituicao = None
        self.cnpj_operadora = ''
        self.aut_cartao = ''
        self.qrcod_cart_digital = ''
        self.cod_carteira_digital = ''
        self.nome_carteira_digital = ''
        self.nome_instituicao = None
        self.mod_bandeira = None
        self.mod_parcelas = 1

        #------------- campos para socio torcedor(concomitante)--------------------
        self.deconto_unitario = 0 # Valor de desconto unitário em centavos
        self.precent_deconto_unitario = 0 # Percentual de desconto unitário
        self.qtd_prod_para_desconto = 0 # Qtd max de produtos em que o desconto pode ser aplicado
        self.limite_total_desconto = 0 # Valor de desconto limite total em centavos que pode ser consedido

    def __repr__(self):
        return '<__class__.SigTef %s, %s, %s>' % (self.endereco_ip.value, self.codigo_loja.value,
                                                  self.identificacao_terminal.value)

    def _configura_sitef(self):
        retval = self.lib_clisitef.ConfiguraIntSiTefInterativoEx(self.endereco_ip,
                                                                 self.codigo_loja,
                                                                 self.identificacao_terminal,
                                                                 self.configura_resultado,
                                                                 self.parms_adicionoais)
        return retval

    @Log
    def continue_(self, buffer):
        if self._cbarras:
            buffer = '1:%s' % buffer
            self._cbarras = False
        self.buffer = c_buffer(buffer, 20000)
        self._stop_event = False
        self._continua_funcao()

    @Log
    def inicia_sitef(self, valor, numero_cupom_fiscal, data_fiscal, horario,
                     operador, modalidade=0, restricoes=''):

        self.mod_parcelas = 1
        if not self._status_iniciou_sitef:
            self._status_iniciou_sitef = True
            self.bloqueio = True
            self.cnpj_operadora = ''
            self.aut_cartao = ''
            self.qrcod_cart_digital = ''
            valor = '%.2f' % valor
            valor = valor.replace('.', ',')
            self.flag_recarga = modalidade
            self.modalidade = c_int(modalidade)
            self.valor = c_buffer(str(valor), len(str(valor)))
            self.numero_cupom_fiscal = c_buffer(str(numero_cupom_fiscal),
                                                len(str(numero_cupom_fiscal)))
            self.data_fiscal = c_buffer(data_fiscal, len(data_fiscal))
            self.horario = c_buffer(horario, len(horario))
            self.operador = c_buffer(operador, len(operador))
            self.restricoes = c_buffer(str(restricoes), len(str(restricoes)))
            self.data_str = data_fiscal
            self.hora_str = horario
            self.continua_navegacao.value = 0
            cpf = self.parent.cliente.supcli_cnpj_cpf
            if config['usa_qrcode_tela']:
                # o qrcode serah devolvido para ser apresentado na tela pela aplicacao
                par = '{DevolveStringQRCode=1}'
                self.restricoes = c_buffer(par, len(par))

            # IniciaFuncao
            # if not cpf or self.parent.cliente.supcli_tipo_pessoa == u'J':
            #     # self.restricoes = c_char_p('{DevolveStringQRCode=1}')
            #     pass
            # else:  # cpf informado anterior
            #     # todo: verificar quando nao estiver usando Carteira digital, ou se o CPF incial nao eh o do pagamento
            #     par_cpf = c_char_p('{DevolveStringQRCode=1;CPFColetado=%s;}' %
            #                        self.parent.cliente.supcli_cnpj_cpf[-11:])

            retval = self.lib_clisitef.IniciaFuncaoSiTefInterativo(self.modalidade,
                                                                   byref(self.valor),
                                                                   byref(self.numero_cupom_fiscal),
                                                                   byref(self.data_fiscal),
                                                                   byref(self.horario),
                                                                   byref(self.operador),
                                                                   byref(self.restricoes))
            if retval == 10000:
                self._stop_event = False
                self._continua_funcao()
            elif retval in ERR_COD.keys():
                raise Exception, ERR_COD[retval]
            else:
                raise Exception, 'Erro interno'
        else:
            retval = self._status_iniciou_sitef
        return retval

    @Log
    def consulta_registra_produtos_ambev(self):
        now = datetime.now()
        data_fiscal = now.strftime('%Y%m%d')
        horario = now.strftime('%H%M%S')
        valor = '0,00'
        operador = self.parent.user_logged.name
        restricoes=''
        self.modalidade = c_int(64)
        self.valor = c_buffer(str(valor), len(str(valor)))
        numero_cupom_fiscal = self.parent.mov_capa.supcap_nro_cupom
        self.numero_cupom_fiscal = c_buffer(str(numero_cupom_fiscal),
                                            len(str(numero_cupom_fiscal)))
        self.data_fiscal = c_buffer(data_fiscal, len(data_fiscal))
        self.horario = c_buffer(horario, len(horario))
        self.operador = c_buffer(operador, len(operador))
        self.restricoes = c_char_p(restricoes)
        self.data_str = data_fiscal
        self.hora_str = horario
        # IniciaFuncao
        retval = self.lib_clisitef.IniciaFuncaoSiTefInterativo(self.modalidade,
                                                               byref(self.valor),
                                                               byref(self.numero_cupom_fiscal),
                                                               byref(self.data_fiscal),
                                                               byref(self.horario),
                                                               byref(self.operador),
                                                               byref(self.restricoes))
        if retval == 10000:
            self._stop_event = False
            self._continua_funcao()
        elif retval in ERR_COD.keys():
            raise Exception, ERR_COD[retval]
        else:
            raise Exception, 'Erro interno'

        return retval

    @Log
    def _continua_funcao(self):
        if not self.bloqueio:
            self._status_iniciou_sitef = False

        self.bloqueio = False
        if self.parent:
            self.parent.infoText('Aguarde...', False)
        while not self._stop_event:
            QtGui.QApplication.processEvents()
            if self.parent.cancel_pinpad:
                self.parent.cancel_pinpad = False
                # o valr -1 cancela a operacao que está sendo feita no Tef
                self.continua_navegacao.value = -1 # parada forcada pelo operador usando o botao na tela

            retval = self.lib_clisitef.ContinuaFuncaoSiTefInterativo(
                byref(self.proximo_comando), byref(self.tipo_campo),
                byref(self.tam_minimo), byref(self.tam_maximo),
                byref(self.buffer), sizeof(self.buffer),
                self.continua_navegacao)

            msg_info = 'modalidade: %d, retorno: %d, proximo_comando: %d, tipo_campo: %d, ' \
                       'tam_minimo: %d, tam_maximo: %d, buffer: %s, tam_max_buffer: %d, ' \
                       'navegacao: %d' % (self.modalidade.value, retval, self.proximo_comando.value,
                                          self.tipo_campo.value,
                                          self.tam_minimo.value, self.tam_maximo.value,
                                          self.buffer.value,
                                          self.tam_max_buffer.value,
                                          self.continua_navegacao.value)

            self.continua_navegacao.value = 0

            Log.info(msg_info)
            str_buffer = self.buffer.value
            self.buffer = c_buffer('', 20000)
            proximo_commando = self.proximo_comando.value
            tipo_campo = self.tipo_campo.value

            if retval == 0:
                if self.gui:
                    self.parent.show_info_proccess(False)
                    if self.modalidade.value == 0:
                        if not self.desconto_socio_registrado:
                            valor = self.valor.value
                            valor = Decimal(valor.replace(',', '.'))
                            if not self._consulta_cheque:
                                if self.mod_parcelas > 1:
                                    parcelas_tef = round(valor/Decimal(self.mod_parcelas),2)
                                    residuo = round(valor - Decimal(parcelas_tef)*Decimal(self.mod_parcelas),2)
                                    primeira_parcela = 0
                                    if residuo < -0.001 or 0.001 < residuo:
                                        primeira_parcela = parcelas_tef + residuo

                                    msg = 'Compra Parcelada com Cartão\n' + \
                                          'Valor Total: R$%s\n' % valor + \
                                          'Quantidade Parcelas: %s\n' % self.mod_parcelas
                                    for i in range(1, self.mod_parcelas +1):
                                        if primeira_parcela:
                                            msg += 'Parcela %s - Valor: R$%s\n' % (i, primeira_parcela)
                                            primeira_parcela=0
                                        else:
                                            msg += 'Parcela %s - Valor: R$%s\n' % (i, parcelas_tef)
                                
                                    message_dlg(self.parent, MSG_INFO, msg)

                                finalizadora = self.get_finalizadora_tef()
                                if not finalizadora:
                                    Log.error(
                                        "Supervisor OFFLINE! Utilizando finalizadora padrao 9999!")
                                    finalizadora = CaixaCadFinalizadora.get_by(
                                                        supfin_codigo='9999')
                                self.parent.obj_cupom_fiscal.add_payment(
                                    valor, checa_tef=False,
                                    pfinalizadora=finalizadora,
                                    parcelas=self.mod_parcelas)
                                self.mod_parcelas = 1
                            else:
                                self.parent._set_screen_state(self.parent._state_screen)
                                self.parent._consulta_cheque = False
                                self.parent.show_info_proccess(False)
                                self._apaga_arquivo()
                    elif self.modalidade.value in [2,3, 122]:
                        valor = self.valor.value
                        valor = Decimal(valor.replace(',', '.'))
                        finalizadora = self.get_finalizadora_tef()
                        if not finalizadora:
                            Log.error(
                                "Supervisor OFFLINE! Utilizando finalizadora padrao 9999!")
                            finalizadora = CaixaCadFinalizadora.get_by(
                                supfin_codigo='9999')
                        self.parent.obj_cupom_fiscal.add_payment(
                            valor, checa_tef=False,
                            pfinalizadora=finalizadora,
                            parcelas=self.mod_parcelas)

                    elif self.modalidade.value == 61:
                        self.parent.ui.lblCliente.setText(u'Sócio  ')
                        self.parent.ui.lblImgCliente.setPixmap(QtGui.QPixmap(
                                     ":/logos/images/logos/socio_torcedor.png"))
                        self._grava_arq_socio_torcedor()
                        self.parent._set_screen_state(self.parent._state_screen)
                    elif self.modalidade.value == 64:
                        pass

                    # elif self.modalidade.value == 122:
                    #     valor = self.valor.value
                    #     valor = Decimal(valor.replace(',', '.'))
                    #     finalizadora = CaixaCadFinalizadora.get_by(name=CARTEIRA_DIGITAL.get(self.cod_carteira_digital).upper())
                    #     self.parent.obj_cupom_fiscal.add_payment(valor, pfinalizadora=finalizadora)

                    elif self.modalidade.value == 200:
                        self.finalizar_transacao(1, self.numero_cupom_fiscal.value,
                                                self.data_fiscal.value, self.horario.value)
                        self.parent._set_screen_state(self.parent._state_screen)

                    elif self.modalidade.value == 300:
                        self.finalizar_transacao(1, self.numero_cupom_fiscal.value,
                                                self.data_fiscal.value, self.horario.value)
                        if config['usagaveta']:
                            self.parent.obj_impressoras.supcci_imprimir("{gaveta}")

                        self.parent._set_screen_state(self.parent._state_screen)

                    else:
                        self.parent._set_screen_state(self.parent._state_screen)
                        # parent.setControlFocus(parent._CONTROL_FOCUS, parent.ui.menuTef)
                else:
                    Log.info('*** FIM ***')
                self._grava_arquivo()
                self._stop_event = True

            elif retval == 10000:
                if proximo_commando == 0:
                    self.valor_tipo_campo(tipo_campo, str_buffer)

                elif proximo_commando in [1, 2, 3]:  # Apresentar Mensagem no Visor
                    self._ultima_msg = str_buffer
                    QtGui.QApplication.processEvents()
                    if self.parent:
                        self.parent.infoText('%d - %s' % (proximo_commando, str_buffer), False)
                    if self.gui:
                        self.parent.teclado and self.parent.teclado.envia_display(str_buffer[:80])
                        self.parent.show_info_proccess(True, str_buffer, tef=True)
                        QtGui.QApplication.processEvents()
                    else:
                        Log.info('Próximo comando: ' + str(str_buffer))

                elif proximo_commando == 4:  # Texto utilizado no menu
                    self.parent.infoText(str_buffer, False)
                    if self.gui:
                        self.parent.ui.lblSigTefInfo.setText(str_buffer)
                    else:
                        msg = '%s' % str_buffer
                        Log.info('=' * len(msg))
                        Log.info(str(msg))
                        Log.info('-' * len(msg))

                elif proximo_commando in [11, 12, 13, 14]:  # Limpar Mensagem
                    self.parent.infoText('', False)
                    self.parent.show_info_proccess(False)

                elif proximo_commando == 20:  # Mensagem Sim/Não
                    if self.gui:
                        ret_msg = message_dlg(self.parent, MSG_QUESTION, str_buffer)
                        if ret_msg == YES_ROLE:
                            ret = 0
                        else:
                            ret = 1
                    else:
                        ret = 1
                        r = raw_input('Sim/Nao: ')
                        if r in ['S', 's', 'Y', 'y']:
                            ret = 0

                    self.buffer = c_buffer(str(ret), 20481)

                elif proximo_commando == 21:  # Apresentar/Montar Menu
                    if not self.gui:
                        for iten in str_buffer[0:-1].split(';'):
                            Log.info(iten)
                        self.continue_(raw_input('Digite uma opcao: '))
                    else:
                        self.parent.show_info_proccess(False)
                        tableSigTefMenu = self.parent.ui.tableSigTefMenu

                        # parent.ui.chkSigTef.setChecked(False)

                        rows_count = tableSigTefMenu.rowCount()
                        if rows_count > 0:
                            rows_c = range(0, rows_count + 1)
                            rows_c.reverse()
                            for r in rows_c:
                                tableSigTefMenu.removeRow(r)

                        # POPULA O WIDGET TABLE COM O RESULTADO DA BUSCA
                        rows = 0
                        pix = 0
                        for iten in str_buffer[0:-1].split(';'):
                            idx, desc = iten.split(':')
                            tableSigTefMenu.insertRow(rows)
                            # CNPJ/CPF
                            iOpcao = QtGui.QTableWidgetItem(idx)
                            iOpcao.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                            # Quantidade
                            iDescricao = QtGui.QTableWidgetItem(desc)
                            iDescricao.setFlags(
                                QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                            # Adiciona as linhas no Widget
                            tableSigTefMenu.setItem(rows, 0, iOpcao)
                            tableSigTefMenu.setItem(rows, 1, iDescricao)
                            if desc.upper() == 'PIX':
                                pix = rows
                            rows += 1

                        if config['autoatende']:
                            self.parent._returnPressed_tableSigTefMenu(pix)
                        else:
                            tableSigTefMenu.selectRow(0)
                            self.parent.ui.lblSigTefInfo.setVisible(True)
                            self.parent.ui.tableSigTefMenu.setVisible(True)
                            self.parent.setControlFocus(self.parent.ui.tableSigTefMenu,
                                                        self.parent.ui.tableSigTefMenu)
                            self._stop_event = True

                elif proximo_commando == 22:  # Notificar Operador
                    if self.gui:
                        self.parent.infoText(str_buffer, False)
                        message_dlg(self.parent, MSG_INFO, str_buffer)
                    else:
                        print 'Confirme a operação'
                        raw_input(str_buffer)
                        # msgbox_information(str_buffer)
                        # parent.show_info_proccess(False)
                        # parent.messagebox('SigPDV', str_buffer)
                        # self.buffer = c_buffer('', 20000)

                elif proximo_commando == 23:
                    if self.gui:
                        msg = '%s<span style=" color:#ff0000;"><br><br><b>Pressione SAIR para cancelar.</b>' % \
                              self._ultima_msg
                        self.parent.btn_cancel_info_dialog = True
                        close = self.parent.show_info_proccess(True, msg)
                        # if close:
                        # self.continua_navegacao = c_int(-1)

                elif proximo_commando == 29:
                    self.envia_dados(tipo_campo, str_buffer)

                elif proximo_commando == 31:  # Solicitar dados do cheque a ser consultado
                    if self.gui:
                        self.parent.show_info_proccess(False)
                        self.parent.ui.stackedActions.setCurrentIndex(11)
                        self.parent.ui.lblSigTefInfo.setText(str_buffer)
                        self.parent.setControlFocus(self.parent.ui.edtSigTefTexto,
                                                    self.parent.ui.edtCompensacao)
                        self.parent.infoText(str_buffer, False)
                        self._consulta_cheque = True
                        self._stop_event = True

                elif proximo_commando in [30, 35]:  # Exibir Caixa de Texto
                    if tipo_campo == 500:
                        if self.gui:
                            self.parent.show_info_proccess(False)
                            if not self.parent.user_logged.supope_operador_supervisor \
                                    and not dlg_pwd(self.parent, msg_info='SigTEF').exec_():
                                    self.continua_navegacao = c_int(-1)
                        else:
                            r = raw_input('Digite a senha do supervisor: ')
                            if not r:
                                self.continua_navegacao = c_int(-1)
                        continue

                    if proximo_commando == 35:
                        self._cbarras = True
                    if self.gui:
                        self.parent.show_info_proccess(False)
                        self.parent.ui.lblSigTefInfo.setVisible(True)
                        self.parent.ui.edtSigTefTexto.setVisible(True)
                        self.parent.ui.edtSigTefTexto.setMaxLength(self.tam_maximo.value)
                        self.parent.ui.lblSigTefInfo.setText(str_buffer)
                        self.parent.setControlFocus(self.parent.ui.edtSigTefTexto,
                                                    self.parent.ui.edtSigTefTexto)
                        self.parent.infoText(str_buffer, False)
                        self._stop_event = True
                    else:
                        msg = '%s: Max: %d' % (str_buffer, self.tam_maximo.value)
                        # print '=' * len(msg)
                        # print msg
                        # print '-' * len(msg)
                        self.continue_(raw_input(msg))
                elif proximo_commando == 34:  # Exibir Valor Monetário
                    if self.gui:
                        self.parent.show_info_proccess(False)
                        self.parent.ui.lblSigTefInfo.setVisible(True)
                        self.parent.ui.spinSigTefValor.setVisible(True)
                        self.parent.ui.lblSigTefInfo.setText(str_buffer)
                        self.parent.setControlFocus(self.parent.ui.edtSigTefTexto,
                                                    self.parent.ui.spinSigTefValor)
                        self.parent.infoText(str_buffer, False)
                        self._stop_event = True
                    else:
                        msg = '%s' % str_buffer
                        print '=' * len(msg)
                        print msg
                        print '-' * len(msg)
                        self.continue_(raw_input('Digite um valor R$: '))
                elif proximo_commando == 50:  # mostrar qrcode para carteiras digitais/Pix
                    QtGui.QApplication.processEvents()
                    if tipo_campo == 584:
                        self.parent.teclado and self.parent.teclado.envia_display('Aguardando leitura QRCode...')
                        self.parent.btn_cancel_info_dialog = True
                        self.parent.show_info_proccess(True, 'QRCODE' + str_buffer)
                        QtGui.QApplication.processEvents()

                elif proximo_commando == 51: # limpar qrcode para carteiras digitais/Pix
                    self.qrcod_cart_digital = ''
                    self.parent.show_info_proccess(False)
                    self.parent.infoText('', True)
                elif proximo_commando == 52: # mensagem de rodape para carteiras digitais/Pix
                    self.parent.infoText("Aguardando leitura do QRCode...", True)

            elif retval > 0:
                message_dlg(self.parent, MSG_INFO, '0.Negada pelo autorizador\n%s' % str_buffer)
                self.limpa_variaveis()
                if self.modalidade.value == 0 and not self.parent._consulta_cheque:
                    self.parent._set_screen_state(8)

                elif self.modalidade.value == 0 and self.parent._consulta_cheque:
                    self.parent._set_screen_state(self.parent._state_screen)

                else:
                    self.parent._set_screen_state(self.parent._state_screen)
                self._stop_event = True
                self.parent.show_info_proccess(False)

            elif retval < 0:
                msg = ERR_COD[
                    retval] if retval in ERR_COD.keys() \
                            else "Erros detectados internamente pela rotina."
                if self.gui:
                    self.parent.show_info_proccess(False)
                    message_dlg(self.parent, MSG_INFO, "%s" % (msg))
                    Log.info("(%d) %s" % (retval, msg))
                    self.limpa_variaveis()
                    if self.modalidade.value == 0 and not self.parent._consulta_cheque:
                        self.parent._set_screen_state(8)
                        self.parent.ui.lblInfo.setText('')

                    elif self.modalidade.value == 0 and self.parent._consulta_cheque:
                        self.parent._set_screen_state(self.parent._state_screen)
                    elif self.modalidade.value in [2, 3, 122]:
                        self.parent._set_screen_state(self.parent._state_screen)

                    else:
                        self.parent.KEY_ESCAPE_Click()
                else:
                    print '*** THE END ***'
                    print msg
                self._stop_event = True

    @Log
    def envia_dados(self, tipo_campo, str_buffer):
        '''
        Envia dados para o Sitef
        '''
        if tipo_campo == 133:  # NSU do SiTef
            # self.continue_(str(self.tbl_nsu.supnsu_tef_nsu))
            self.buffer = c_buffer(str(self.parent._tef_nsu_socio_torcedor), 20000)

        elif tipo_campo == 1115:  # Lista de produtos vendidos
            # self.continue_(str(self.parent.str_lista_itens_socio))
            self.buffer = c_buffer(str(self.parent.str_lista_itens_socio), 20000)
        # elif tipo_campo == 4153: # Código do PSP (Prestador de Serviços de Pagamento).
        #     self.buffer = c_buffer(' ', 20000)
        # elif tipo_campo == 4154: #
        #     self.buffer = c_buffer(' ', 20000)


    @Log
    def valor_tipo_campo(self, tipo_campo, str_buffer):
        Log.info("================= str_buffer: '%s'" % str_buffer)
        if tipo_campo == 100:  # Modalidade de pagamento formato XXNN
            self.mod_pgto_grupo = str_buffer[:2]
            Log.info("mod_pgto_grupo: '%s'" % str_buffer[:2])
            self.mod_pgto_subgrupo = str_buffer[2:]
            Log.info("mod_pgto_subgrupo: '%s" % str_buffer[2:])

            if self.mod_pgto_grupo in ['01', '02', '03', '99']:
                self.tipo_finalizadora = 'C'

            elif self.mod_pgto_grupo == '00':
                self.tipo_finalizadora = 'H'

            elif self.mod_pgto_grupo == '98':
                self.tipo_finalizadora = 'D'

        elif tipo_campo == 101:  # Nome por extenso da Modalidade de pagamento
            self.nome_finalizadora = str_buffer
        elif tipo_campo == 106: # Codigo da carteira digital
            self.cod_carteira_digital = str_buffer
        elif tipo_campo == 107:
            self.nome_carteira_digital = str_buffer

        elif tipo_campo == 120:  # Contem a linha de autorizacao do cheque de 15 posicoes no maximo.
            self.nsu = str_buffer
            message_dlg(self.parent, MSG_INFO, '%s' % str_buffer)

        elif tipo_campo == 121:  # primeira via
            if self.modalidade.value in (0,2,3, 122):
                self.parent._texto_comprovante = str_buffer.replace('{.', '').replace('}', '')
            else:
                value = self.get_total_cupom(str_buffer)
                cupom_vinculado = None

                if self.modalidade.value == 300:  # Recarga Celular
                    obj_recarga_cel = RecargaCelular(self.parent)
                    obj_recarga_cel.totalizar_recarga(Decimal(value), self.tipo_finalizadora)
                    comp_pagamento = self.parent.obj_cupom_fiscal.get_texto_cupom(str_buffer)
                    self.parent.obj_impressoras.supcci_imprimir(comp_pagamento)
                    return

                elif self.modalidade.value == 112:  # Reimpressão
                    cupom_vinculado = self.get_cupom_vinculado(self.parent.loja.supcfc_loja,
                                                               self.parent.caixa_id,
                                                               '%s' % datetime.strptime(self.data_fiscal.value, '%Y%m%d').date(),
                                                               self.get_total_cupom(str_buffer))
                if cupom_vinculado:
                    self.parent.mov_capa = self.parent.obj_mov_capa.search({'id': cupom_vinculado})

                comp_pagamento = self.parent.obj_cupom_fiscal.get_texto_cupom(str_buffer)
                self.parent.obj_cupom_fiscal.cria_capa_cupom_gerencial(valor=value, tipo='T',
                                                                       comprovante=True)
                self.parent.mov_capa = False
                print '\n\nReimpressão de comprovante de pagamento\n\n', comp_pagamento
                self.parent.obj_impressoras.supcci_imprimir(comp_pagamento)

        elif tipo_campo == 122:  # segunda via
            if self.modalidade.value in (0,2,3, 122):
                if int(config['qtdvias'] or 0) > 1:
                    self.parent._texto_comprovante += '\n{corta}\n'
                    # self.parent._texto_comprovante += '\n----------------------------------------------\n'
                    self.parent._texto_comprovante += str_buffer.replace('{.', '').replace('}', '')
            else:
                self.parent.infoText('Imprimindo relatório gerencial', False)
                if self.modalidade.value != 300:
                    comp_pagamento = self.parent.obj_cupom_fiscal.get_texto_cupom(str_buffer)
                    self.parent.obj_cupom_fiscal.cria_capa_cupom_gerencial(valor=self.get_total_cupom(str_buffer),
                                                                           tipo='T', comprovante=True)
                    print '\n\nReimpressão de comprovante de pagamento\n\n', comp_pagamento
                    self.parent.obj_impressoras.supcci_imprimir(comp_pagamento)
                else:
                    comp_pagamento = self.parent.obj_cupom_fiscal.get_texto_cupom(str_buffer)
                    self.parent.obj_impressoras.supcci_imprimir(comp_pagamento)

                self.parent.mov_capa = False
            Log.info("cod_instituicao: '%s'" % str_buffer)
            self.cod_instituicao = str_buffer

        elif tipo_campo == 132:  # Indice do Cartão (Bandeira)
            Log.info("mod_bandeira: '%s'" % str_buffer)
            self.mod_bandeira = str_buffer

        elif tipo_campo == 133:  # NSU do SiTef
            self._tef_nsu_socio_torcedor = self.tbl_nsu.supnsu_tef_nsu = str_buffer
            try:
                if exec_sql("""
                    SELECT supnsu_data_cupom
                    FROM caixa_cor_nsu
                    WHERE supnsu_data_cupom = '{data_cupom}'
                      AND supnsu_loja = {loja}
                      AND supnsu_caixa = {caixa}
                      AND supnsu_nro_cupom_venda = '{nro_cupom}' """.
                         format(loja=config['loja'],
                                caixa=config['caixa'],
                                nro_cupom=self.numero_cupom_fiscal.value,
                                data_cupom=self.data_fiscal.value), 'C'):

                    sql = """UPDATE caixa_cor_nsu
                             SET supnsu_tef_rede = '{tef_rede}',
                                 supnsu_tef_data_transacao = '{tef_data}',
                                 supnsu_tef_hora_transacao = '{tef_hora}'
                          """.format(tef_rede=self.endereco_ip.value,
                                     tef_data=self.data_fiscal.value,
                                     tef_hora=self.horario.value)

                else:
                    sql = """
                    INSERT INTO  caixa_cor_nsu
                                (supnsu_loja,
                                 supnsu_caixa,
                                 supnsu_nro_cupom_venda,
                                 supnsu_data_cupom,
                                 supnsu_tef_rede,
                                 supnsu_tef_data_transacao,
                                 supnsu_tef_hora_transacao)
                    VALUES      ({loja}, {caixa}, '{nro_cupom}', '{data_cupom}', '{tef_rede}', '{tef_data}', '{tef_hora}');
                    """.format(loja=config['loja'],
                               caixa=config['caixa'],
                               nro_cupom=self.numero_cupom_fiscal.value,
                               data_cupom=self.data_fiscal.value,
                               tef_rede=self.endereco_ip.value,
                               tef_data=self.data_fiscal.value,
                               tef_hora=self.horario.value)
                exec_sql(sql, 'C')
            except Exception as e:
                print e

        elif tipo_campo == 134:  # NSU do Host Autorizador
            self.tbl_nsu.supnsu_tef_finalizacao = str_buffer

        elif tipo_campo == 135:  # codigo de autorizacao para cartao de credito
            self.aut_cartao = str_buffer

        elif tipo_campo == 156:
            Log.info("nome_instituicao: '%s'" % str_buffer)
            self.nome_instituicao = str_buffer
        elif tipo_campo == 584:
            self.qrcod_cart_digital = str_buffer

        elif tipo_campo == 950:  # CNPJ da fonte pagadora (autorizador do cartão)
            self.cnpj_operadora = str_buffer

        elif tipo_campo == 951:  # Bandeira NFCE
            self.cod_bandeira = str_buffer

        # elif tipo_campo == 952:  # Número de autorização NFCE
        #     self.aut_nfce = str_buffer

        # elif tipo_campo == 953:  # Código da credenciadora

        elif tipo_campo == 1115:  # Lista de produtodos vendidos
            self.parent.lista_itens_desconto_socio += str_buffer

        #------------- campos para socio torcedor--------------------
        elif tipo_campo == 4029: # Valor de desconto unitário em centavos
            self.deconto_unitario = str_buffer

        elif tipo_campo == 4008: # Percentual de desconto unitário
            self.precent_deconto_unitario = str_buffer

        elif tipo_campo == 1013: # Qtd max de produtos em que o desconto pode ser aplicado
            self.qtd_prod_para_desconto = str_buffer

        elif tipo_campo == 4039: # Valor de desconto limite total em centavos que pode ser consedido
            self.limite_total_desconto = str_buffer


    @Log
    def get_total_cupom(self, str_buffer):
        value = 0
        for x in str_buffer.split('\n'):
            try:
                if x.upper().startswith('VALOR') or (x.upper().startswith('DO')
                                                  and 'VALOR' in x.upper()) or 'VALOR' in x.upper():
                    value = x.split()[-1].replace('.', '').replace(',', '.').replace('R$', '')
                    value = value.split('-')[0]
                    break
            except Exception as e:
                Log.error('Erro ao pegar o valor da transacao: %s' % e)
                break


        return value

    @Log
    def get_cupom_vinculado(self, loja_id, caixa_id, data_cupom, total_cupom):
        sql = """
        SELECT capa.id
        FROM   caixa_mov_capa             AS capa
        INNER JOIN caixa_mov_fechamentos  AS fec ON fec.supfec_capa_id = capa.id
        INNER JOIN caixa_cad_finalizadora AS fin ON fin.supfin_codigo = fec.supfec_finalizadora
        WHERE  capa.supcap_loja = %s
               AND capa.supcap_caixa = %s
               AND capa.supcap_data_cupom = '%s'
               AND capa.supcap_total_cupom = %s
               AND fin.supfin_usa_tef = true
        ORDER  BY  capa.id DESC
        LIMIT  1
        """ % (loja_id, caixa_id, data_cupom, Decimal(total_cupom))
        id = exec_sql(sql, 'C', ret=True)
        try:
            id = id[0].get('id')
        except:
            id = None

        return id

    @Log
    def finalizar_transacao(self, confirma, numero_cupom_fiscal, data_fiscal, horario,
                            cancela_menu_tef=False):
        '''
        :param confirma:
        :param numero_cupom_fiscal:
        :param data_fiscal:
        :param horario:
        :param cancela_menu_tef: True para cancelar apenas o pagamento via tef mas nao cancela
                                 o socio torcedor
        :return:
        '''
        confirma = c_int(confirma)
        self.desconto_socio_registrado = False
        self.parent.ui.lblCliente.setText('Cliente')
        self.parent.ui.lblImgCliente.setPixmap(QtGui.QPixmap(
                     ":/16x16/art/scalable/actions/contact-new.svg"))
        self.numero_cupom_fiscal = c_buffer(str(numero_cupom_fiscal), len(str(numero_cupom_fiscal)))
        self.data_fiscal = c_buffer(data_fiscal, len(data_fiscal))
        self.horario = c_buffer(horario, len(horario))
        self.mod_parcelas = 1
        print 'FINALIZA TEF', confirma, numero_cupom_fiscal, data_fiscal, horario
        Log.info("Plataforma: %s" % platform.machine())
        if platform.machine() == 'x86_64':
            retval = self.lib_clisitef.FinalizaFuncaoSiTefInterativo(confirma,
                                                                    byref(self.numero_cupom_fiscal),
                                                                    byref(self.data_fiscal),
                                                                    byref(self.horario))
        else: # para as versoes 32 bist esta funcao funciona.
            retval = self.lib_clisitef.FinalizaTransacaoSiTefInterativo(confirma,
                                                                    byref(self.numero_cupom_fiscal),
                                                                    byref(self.data_fiscal),
                                                                    byref(self.horario))

        if not cancela_menu_tef:
            self._apaga_arquivo()
            if self.parent.ler_arq_socio_torcedor():
                self.deleta_arq_socio_torcedor()

            self.limpa_variaveis()

        return retval

    @Log
    def limpa_variaveis(self):
        self.bloqueio = False
        self.usando_sitef = False
        self._status_iniciou_sitef = False
        self.desconto_socio_registrado = False
        self.parent._tef_nsu_socio_torcedor = False
        self.cod_instituicao = None
        self.aut_cartao = ''
        self.qrcod_cart_digital = ''
        self.cod_carteira_digital = ''
        self.nome_carteira_digital = ''
        self.cnpj_operadora = ''
        self.nome_instituicao = None

    @Log
    def consulta_qtde_desconto_socio_torcedor(self, codigo_produto, qtde, nsu_tef, valor_unitario):
        retval = self.lib_clisitef.ConsultaQtdeDescontoSocioTorcedor(codigo_produto,
                                                                     qtde, nsu_tef,
                                                                     valor_unitario)
        if retval == 10000:
            self._stop_event = False
            self._continua_funcao()
        elif retval in ERR_COD.keys():
            raise Exception, ERR_COD[retval]
        else:
            raise Exception, 'Erro interno'

        if not self.parent.desconto_socio_torcedor.get(codigo_produto):
            if self.deconto_unitario or self.precent_deconto_unitario: # se houver desconto de socio torcedor
                self.parent.desconto_socio_torcedor[codigo_produto] = {}

        if self.deconto_unitario:
            if not self.parent.desconto_socio_torcedor.get(codigo_produto).get('desconto'):
                self.parent.desconto_socio_torcedor[codigo_produto]['desconto'] = self.deconto_unitario
            else:
                self.parent.desconto_socio_torcedor[codigo_produto]['desconto'] += self.deconto_unitario

        if self.precent_deconto_unitario:
            if not self.parent.desconto_socio_torcedor.get(codigo_produto).get('precent_desconto'):
                self.parent.desconto_socio_torcedor[codigo_produto]['precent_desconto'] = self.precent_deconto_unitario

        if self.qtd_prod_para_desconto:
            if not self.parent.desconto_socio_torcedor.get(codigo_produto).get('qtd_prod_desconto'):
                self.parent.desconto_socio_torcedor[codigo_produto]['qtd_prod_desconto']= self.qtd_prod_para_desconto

        if self.limite_total_desconto:
            if not self.parent.desconto_socio_torcedor.get(codigo_produto).get('limite_desconto'):
                self.parent.desconto_socio_torcedor[codigo_produto]['limite_desconto'] = self.limite_total_desconto
            self.limite_total_desconto

        return retval

    @Log
    def registra_desconto_socio_torcedor(self, coo, data_fiscal, horario, nome_operador, nsu_socio,
                                                                 lista_produtos, sequencia):
        retval = self.lib_clisitef.RegistraDescontoSocioTorcedorSequencial(coo, data_fiscal, horario, nome_operador,
                                                                           nsu_socio, lista_produtos, sequencia)
        self.desconto_socio_registrado = True
        if retval == 10000:
            self._stop_event = False
            self._continua_funcao()
            self.parent.registrei_desconto = True
            self.desconto_socio_registrado = False
        elif retval in ERR_COD.keys():
            raise Exception, ERR_COD[retval]
        else:
            raise Exception, 'Erro interno'

        return retval

    def verifica_pinpad(self):
        return self.lib_clisitef.VerificaPresencaPinPad()

    def _ler_arquivo(self, file='transaction.dat', finaliza=False, op=False):

        ret = False
        if os.path.exists(file):
            f = open(file, 'r')
            l = f.readline().split('|')
            if len(l) != 3:
                raise IOError('Arquivo vazio: %s' % file)
            else:
                for x in l:
                    if not x: raise IOError('Linha vazia: %s' % file)
                if finaliza:
                    self._apaga_arquivo()
                else:
                    return l
        else:
            print 'arquivo não existe'
        return ret

    def _grava_arquivo(self, file='transaction.dat'):
        f = open(file, 'w')
        txt = '%s|%s|%s' % (
            self.numero_cupom_fiscal.value, self.data_fiscal.value, self.horario.value)
        f.write(txt)
        f.close()

    def _grava_arq_socio_torcedor(self, file='arq_socio_torcedor'):
        """
        Método para criação de arquivo de autorização do cliente no modulo socio torcedor.
        @parament file type str
        """

        f = open(file, 'w')
        txt = 'nsu=%s' % self._tef_nsu_socio_torcedor
        f.write(txt)
        f.close()
        self._tef_nsu_socio_torcedor = None

    def deleta_arq_socio_torcedor(self, file='arq_socio_torcedor'):
        try:
            os.remove(file)
        except Exception as e:
            Log.error("Erro ao deletar arquivo do socio torcedor: %s" % str(e))

    def _apaga_arquivo(self, file='transaction.dat'):
        sql = "DELETE FROM caixa_cor_nsu"
        try:
            exec_sql(sql, 'C')
        except Exception as e:
            Log.info('Erro: %s' % str(e))
        ret = False
        if os.path.exists(file):
            print os.system('rm %s' % file)
            ret = True
        return ret

    @Log
    def get_finalizadora_tef(self):
        """
        Cadastro automático de finalizadoras.

        :return: query CaixaCadFinalizadora
        :rtype object
        """
        res = False

        if not self.mod_bandeira or self.mod_bandeira == '00000':
            Log.error(">>>>>>>> Cartão sem mod_bandeira! <<<<<<<<")

        self.conexao = self.parent.conectado
        self.finalizadora_id = False
        tipo_bandeira = TIPO_BANDEIRA.get(int(self.mod_pgto_grupo))
        if self.nome_finalizadora == 'Carteira Digital':
            tipo_bandeira = 'O'
            self.mod_bandeira = self.cod_carteira_digital

        caixa_bandeira_id = self.get_bandeira_id(self.mod_bandeira, self.mod_pgto_grupo, 'caixa')
        # se nao tem bandeira e nem conexao, retorna False
        if not caixa_bandeira_id and not self.conexao:
            return False

        elif not caixa_bandeira_id: # cadastrar apenas quando estiver conectado com o supervisor
            # consulta no supervisor se ja tem a bandeira
            super_bandeira_id = self.get_bandeira_id(self.mod_bandeira, self.mod_pgto_grupo, 'supervisor')
            if not super_bandeira_id:
                super_bandeira_id = self.cadastrar_bandeira(self.mod_bandeira, tipo_bandeira, 'supervisor')

            Log.info('sup bandeira_id: %s' % super_bandeira_id)
            if not super_bandeira_id:
                return False

            self.finalizadora_id = self.get_finalizadora_id(super_bandeira_id,
                                                            self.tipo_finalizadora,
                                                            self.mod_pgto_grupo,
                                                            self.mod_pgto_subgrupo,
                                                            self.mod_parcelas,
                                                            caixa_supervisor='supervisor')
            if not self.finalizadora_id:
                self.finalizadora_id = self.cadastrar_finalizadora(self.nome_finalizadora,
                                                                   self.tipo_finalizadora,
                                                                   super_bandeira_id,
                                                                   self.mod_pgto_grupo,
                                                                   self.mod_pgto_subgrupo,
                                                                   self.mod_parcelas,
                                                                   'supervisor')

            Log.info('supervisor_cad_finalizadora_id: %s' % self.finalizadora_id)
            # depois de cadastrar a bandeira no supervisor, cadastra no caixa tambem
            caixa_bandeira_id = self.cadastrar_bandeira(self.mod_bandeira, tipo_bandeira, 'caixa')
            self.finalizadora_id = self.cadastrar_finalizadora(self.nome_finalizadora,
                                                               self.tipo_finalizadora,
                                                               caixa_bandeira_id,
                                                               self.mod_pgto_grupo,
                                                               self.mod_pgto_subgrupo,
                                                               self.mod_parcelas,
                                                               'caixa')

        if not self.finalizadora_id:
            self.finalizadora_id = self.get_finalizadora_id(caixa_bandeira_id,
                                                            self.tipo_finalizadora,
                                                            self.mod_pgto_grupo,
                                                            self.mod_pgto_subgrupo,
                                                            self.mod_parcelas,
                                                            caixa_supervisor='caixa')
            if not self.finalizadora_id and not self.conexao:
                return False

             # nao havendo finalizadora no caixa, cadastra no supervisor e no caixa
            if not self.finalizadora_id:
                super_bandeira_id = self.get_bandeira_id(self.mod_bandeira, self.mod_pgto_grupo,
                                                       'supervisor')
                self.finalizadora_id = self.get_finalizadora_id(super_bandeira_id,
                                                                self.tipo_finalizadora,
                                                                self.mod_pgto_grupo,
                                                                self.mod_pgto_subgrupo,
                                                                self.mod_parcelas,
                                                                caixa_supervisor='supervisor')
                if not self.finalizadora_id:
                    self.finalizadora_id = self.cadastrar_finalizadora(self.nome_finalizadora,
                                                                       self.tipo_finalizadora,
                                                                       super_bandeira_id,
                                                                       self.mod_pgto_grupo,
                                                                       self.mod_pgto_subgrupo,
                                                                       self.mod_parcelas,
                                                                       'supervisor')

                self.finalizadora_id = self.cadastrar_finalizadora(self.nome_finalizadora,
                                                                   self.tipo_finalizadora,
                                                                   caixa_bandeira_id,
                                                                   self.mod_pgto_grupo,
                                                                   self.mod_pgto_subgrupo,
                                                                   self.mod_parcelas,
                                                                   'caixa')

        Log.info('caixa_cad_finalizadora_id: %s' % self.finalizadora_id)

        if self.finalizadora_id:
            res = CaixaCadFinalizadora.get_by(id=self.finalizadora_id)

        return res

    @Log
    def get_name_finalizadora(self, nome_finalizadora, pgto_grupo, qtd_parcelas=1):
        if self.nome_finalizadora == 'Carteira Digital':
            if self.nome_carteira_digital:
                return nome_finalizadora  + ' - ' + self.nome_carteira_digital

            return nome_finalizadora

        mod_parcela = '%sx' % qtd_parcelas if qtd_parcelas > 1 else ''

        tipo_bandeira = TIPO_BANDEIRA.get(int(pgto_grupo))

        bandeira = CaixaCadBandeiras.get_by(supban_codigo=int(self.mod_bandeira),
                                            supban_tipo=tipo_bandeira)

        if bandeira:
            bandeira = bandeira.name
        else:
            bandeira = self.mod_bandeira

        name = '%s %s - %s' % (nome_finalizadora, mod_parcela, bandeira)
        name = name.replace('  ', ' ').upper()

        return name

    @Log
    def get_finalizadora_id(self, bandeira_id, tipo_finalizadora, pgto_grupo,
                            pgto_sub_grupo, qtd_parcelas, caixa_supervisor='supervisor'):

        cond_parcelas = "AND (supfin_outros_qtd_parcelas IN (0, 1)) "
        if qtd_parcelas > 1:
            cond_parcelas = "AND supfin_outros_qtd_parcelas = %s " % qtd_parcelas

        sql_finalizadora_id = """
            SELECT id, supfin_codigo
            FROM   {caixa_supervisor}_cad_finalizadora
            WHERE  supfin_tipo_finalizadora = '{tipo_finalizadora}'
              AND  supfin_mod_pgto_grupo = '{pgto_grupo}'
              AND  supfin_mod_pgto_sub_grupo = '{pgto_sub_grupo}'
              {cond_parcelas}
              AND  (supfin_bandeira_cre = '{bandeira_id}'
                   OR  supfin_bandeira_deb = '{bandeira_id}')
        """.format(caixa_supervisor=caixa_supervisor,
                   tipo_finalizadora=tipo_finalizadora,
                   pgto_grupo=pgto_grupo,
                   pgto_sub_grupo=pgto_sub_grupo,
                   cond_parcelas=cond_parcelas,
                   bandeira_id=bandeira_id)

        Log.info(sql_finalizadora_id)

        try:
            finalizadora_id = exec_sql(sql_finalizadora_id, caixa_supervisor, ret=True)
        except Exception as e:
            Log.error('%s' % e)
            finalizadora_id = False

        if finalizadora_id and isinstance(finalizadora_id, list):
            finalizadora_id = finalizadora_id[0]

        finalizadora_id = finalizadora_id.get('id') if finalizadora_id else None
        return finalizadora_id

    @Log
    def cadastrar_finalizadora(self, nome_finalizadora, tipo_finalizadora, bandeira_id, pgto_grupo,
                               pgto_sub_grupo, qtd_parcelas, caixa_supervisor='supervisor'):

        nome_finalizadora = self.get_name_finalizadora(nome_finalizadora,
                                                       pgto_grupo, qtd_parcelas)
        query_cad_finalizadora = CaixaCadFinalizadora.get_by(
                                 supfin_tipo_finalizadora=tipo_finalizadora)
        codigo_na_ecf = query_cad_finalizadora.supfin_codigo_na_ecf if query_cad_finalizadora else 0
        codigo_finalizadora = 0
        campo_operadora_id = ''
        operadora_id = ''
        if caixa_supervisor == 'supervisor':
            if self.cnpj_operadora:
                dados_operadora = exec_sql("""SELECT id 
                                              FROM supervisor_cad_operadora_cartao 
                                              WHERE supopc_cnpj = '%s' """ % self.cnpj_operadora, 'S', ret=True)
                if dados_operadora and dados_operadora[0].get('id'):
                    campo_operadora_id = 'supfin_operadora_id,'
                    operadora_id = '%s,' % dados_operadora[0].get('id')

            sql_supfin_codigo = """SELECT Max(supfin_codigo) AS cod
                                   FROM supervisor_cad_finalizadora
                                   WHERE supfin_codigo != 9999 """
            codigo_finalizadora = exec_sql(sql_supfin_codigo, 'S', ret=True)[0]['cod']
            codigo_finalizadora += 1

        elif caixa_supervisor == 'caixa':
            codigo_finalizadora = self.get_supervisor_codigo_finalizadora(self.finalizadora_id)
            dados_fin_sup = exec_sql("""SELECT supfin_operadora_id As op_id
                                        FROM supervisor_cad_finalizadora 
                                        WHERE supfin_codigo = %s""" % codigo_finalizadora, 'S', ret=True)
            if dados_fin_sup and dados_fin_sup[0].get('op_id'):
                campo_operadora_id = 'supfin_operadora_id,'
                operadora_id = '%s,' % dados_fin_sup[0].get('op_id')

            if CaixaCadFinalizadora.get_by(supfin_codigo=codigo_finalizadora):
                sql_del = "DELETE FROM caixa_cad_finalizadora WHERE supfin_codigo = %s" % \
                           codigo_finalizadora
                exec_sql(sql_del, 'C')


        sql = """
            INSERT INTO {caixa_supervisor}_cad_finalizadora
                        (supfin_codigo,
                         name,
                         supfin_tipo_finalizadora,
                         supfin_codigo_na_ecf,
                         supfin_avista_aprazo,
                         supfin_usa_tef,
                         supfin_cartao_tipo,
                         supfin_mod_pgto_grupo,
                         supfin_mod_pgto_sub_grupo,
                         supfin_bandeira_cre,
                         supfin_bandeira_deb,
                         {supfin_operadora_id}
                         supfin_cliente,
                         supfin_outros_qtd_parcelas,
                         supfin_outros_intervalo_parcelas,
                         supfin_outros_prazo_primeira)
            VALUES      ({codigo_finalizadora},
                        '{nome_finalizadora}',
                        '{tipo_finalizadora}',
                         {codigo_na_ecf},
                        '{tipo_pg}',
                        '{usa_tef}',
                        '{debito_credito}',
                        '{pgto_grupo}',
                        '{pgto_sub_grupo}',
                         {bandeira_id_cre},
                         {bandeira_id_deb},
                         {operadora_id}
                         {bandeira_id},
                        '{qtd_parcelas}',
                        '{intervalo_parcelas}',
                        '{prazo_primeira}')
            RETURNING   id
                     """.format(caixa_supervisor=caixa_supervisor,
                                codigo_finalizadora=codigo_finalizadora,
                                nome_finalizadora=nome_finalizadora,
                                tipo_finalizadora=tipo_finalizadora,
                                codigo_na_ecf=codigo_na_ecf,
                                tipo_pg='V' if int(pgto_sub_grupo) in (0, 99) else 'P',
                                usa_tef='T',
                                debito_credito='D' if int(pgto_grupo) == 1 else 'C',
                                pgto_grupo=pgto_grupo,
                                pgto_sub_grupo=pgto_sub_grupo,
                                bandeira_id_cre=bandeira_id if int(pgto_grupo) != 1 else 'Null',
                                bandeira_id_deb=bandeira_id if int(pgto_grupo) != 2 else 'Null',
                                bandeira_id=bandeira_id,
                                qtd_parcelas=qtd_parcelas,
                                supfin_operadora_id=campo_operadora_id,
                                operadora_id=operadora_id,
                                intervalo_parcelas=0 if int(pgto_sub_grupo) in (0, 99) else 31,
                                prazo_primeira=0 if int(pgto_sub_grupo) in (0, 99) else 30)

        Log.info(sql)
        try:
            finalizadora_id = exec_sql(sql, caixa_supervisor, ret=True)
            if finalizadora_id and isinstance(finalizadora_id, list):
                finalizadora_id = finalizadora_id[0]

            finalizadora_id = finalizadora_id.get('id') if finalizadora_id else None

            if config['db_sup_name_master'] and caixa_supervisor == 'supervisor':
                # cria no supervisor uma acao pendente para o supervisor_master caso tenha o uso
                # do mesmo
                sql = """INSERT INTO supervisor_mov_acao_pendente
                                (supeape_loja, supape_tabela, supape_situacao,
                                supape_total_em_aberto_cliente)
                         VALUES ({loja}, 'Finalizadoras', 'A', 0)
                      """.format(loja=config['loja'])
                exec_sql(sql, caixa_supervisor)
        except Exception as e:
            Log.error("Erro ao cadatrar finalizadadora: %s" % str(e))
            finalizadora_id = None

        return finalizadora_id

    @staticmethod
    def get_supervisor_codigo_finalizadora(finalizadora_id):
        sql = """SELECT supfin_codigo
                 FROM supervisor_cad_finalizadora
                 WHERE id=%s
                   AND supfin_codigo != 9999""" % finalizadora_id
        codigo_finalizadora = exec_sql(sql, 'S', ret=True)[0]['supfin_codigo']
        return codigo_finalizadora

    @Log
    def get_bandeira_id(self, mod_bandeira, pgto_grupo, caixa_supervisor='supervisor'):
        tipo_bandeira = TIPO_BANDEIRA.get(int(pgto_grupo))
        if self.nome_finalizadora == 'Carteira Digital':
            tipo_bandeira = 'O'

        sql_bandeira_id = """
            SELECT id
            FROM   %s_cad_bandeiras
            WHERE  supban_codigo = %s
            AND    supban_tipo = '%s'
        """ % (caixa_supervisor, mod_bandeira, tipo_bandeira)

        Log.info(sql_bandeira_id)
        bandeira_id = None
        try:
            bandeira_id = exec_sql(sql_bandeira_id, caixa_supervisor, ret=True)
            if bandeira_id and isinstance(bandeira_id, list):
                bandeira_id=bandeira_id[0]

            bandeira_id = bandeira_id.get('id') if bandeira_id else None
        except Exception as e:
            Log.info('Erro ao consultar bandeira: %s' % str(e))

        return bandeira_id

    @Log
    def cadastrar_bandeira(self, codigo, tipo, caixa_supervisor='supervisor'):
        descricao = codigo.rjust(5, '0')
        if self.nome_finalizadora == 'Carteira Digital':
            descricao = self.nome_carteira_digital or 'Sem Nome'

        if caixa_supervisor == 'caixa':
            sql = """
                INSERT INTO caixa_cad_bandeiras
                            (supban_codigo,
                             name,
                             supban_descricao_completa,
                             supban_tipo)
                VALUES      (%s, '%s', '%s', '%s')
                RETURNING   id
            """ % (codigo, descricao, descricao, tipo)

        elif caixa_supervisor == 'supervisor':
            sql = """
                INSERT INTO supervisor_cad_bandeiras
                            (supban_codigo,
                             name,
                             supban_descricao_completa,
                             supban_tipo)
                VALUES      ({codigo}, '{descricao}', '{descricao}', '{tipo}')
                RETURNING   id
            """.format(codigo=codigo,
                       descricao=descricao,
                       tipo=tipo)

        Log.info(sql)

        try:
            bandeira_id = exec_sql(sql, caixa_supervisor, ret=True)
            if config['db_sup_name_master'] and caixa_supervisor == 'supervisor':
                # cria no supervisor uma acao pendente para o supervisor_master caso tenha o uso
                # do mesmo
                sql = """INSERT INTO supervisor_mov_acao_pendente
                                (supeape_loja, supape_tabela, supape_situacao,
                                supape_total_em_aberto_cliente)
                         VALUES ({loja}, 'Bandeiras', 'A', 0)
                      """.format(loja=config['loja'])
                exec_sql(sql, caixa_supervisor)

        except Exception as e:
            Log.error("Erro ao cadastrar bandeira: %s" % str(e))
            bandeira_id = None

        if bandeira_id and isinstance(bandeira_id, list):
            bandeira_id = bandeira_id[0]

        if isinstance(bandeira_id, dict):
            bandeira_id = bandeira_id.get('id') if bandeira_id else None

        return bandeira_id


if __name__ == "__main__":
    from datetime import datetime

    sig_tef = SigTefI(None)
    now = datetime.now()
    data_fiscal = now.strftime('%Y%m%d')
    horario = now.strftime('%H%M%S')
    modalidade = 110

    sig_tef.inicia_sitef(5.9, '1656', data_fiscal, horario, 'christian', modalidade)

